/*================================================================
  File:     01_CapturePlans.sql

  Summary:  Queries used to show the various Showplan methods

  Latest SQL Server Versions tested: SQL Server 2012 11.0.3401
------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com

  (c) 2016, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
=================================================================*/

USE [AdventureWorks2016];
GO

/*
	Clear cache again - not for production use :(
*/
DBCC FREEPROCCACHE;
GO


/*
	Display Estimated Graphical Showplan
*/
SELECT
[d].[SalesOrderDetailID], [d].[ProductID], [h].[CustomerID], [h].[TerritoryID]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Sales].[SalesOrderHeader] [h] on [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [h].[SalesPersonID] IS NULL;
GO


/*
	proving it's not actually cached...
*/
SELECT	[cp].[size_in_bytes],
		[cp].[cacheobjtype],
		[cp].[objtype],
		[cp].[plan_handle],
		[dest].[text]
FROM [sys].[dm_exec_cached_plans] [cp]
CROSS APPLY [sys].[dm_exec_sql_text]([cp].[plan_handle]) [dest]
WHERE [dest].[text] LIKE '%SalesOrderHeader%';


/*
	Display Actual Graphical Showplan
	For each operator get estimated and actual information
	tool tip is a subset of information...F4 is your friend
	SELECT - memory grant info, estimated subtree cost 
*/

SELECT
[d].[SalesOrderDetailID], [d].[ProductID], [h].[CustomerID], [h].[TerritoryID]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Sales].[SalesOrderHeader] [h] on [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [h].[SalesPersonID] IS NULL;
GO


USE [AdventureWorks2016];
GO

/*
	XML Showplan *Estimated*
	can see operations in XML format, can search (CTRL+F)
	note row estimates
*/

SET SHOWPLAN_XML ON;
GO

SELECT
	[t].[TransactionID],
	[t].[ProductID],
	[h].[OrderDate],
	[h].[CustomerID],
	[h].[TotalDue]
FROM [Production].[TransactionHistory] [t]
JOIN [Sales].[SalesOrderHeader] [h] on [t].[ReferenceOrderID] = [h].[SalesOrderID];
GO

SET SHOWPLAN_XML OFF;
GO


/*
	XML Showplan Actual
	Includes run-time statisics
	note estimates versus actuals
*/
SET STATISTICS XML ON;
GO

SELECT
[d].[SalesOrderDetailID], [d].[ProductID], [h].[CustomerID], [h].[TerritoryID]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Sales].[SalesOrderHeader] [h] on [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [h].[SalesPersonID] IS NULL;
GO

SET STATISTICS XML OFF;
GO


/*
	Save and open a saved .sqlplan file

	View in SQLSentry Plan Explorer

	Copy XML into PE

	Viewing the schema:
	http://schemas.microsoft.com/sqlserver/2004/07/showplan/

*/



/*
	Get a plan from cache...
	clear cache first - not for production use :(
*/
DBCC FREEPROCCACHE;
GO


/*
	execute a query
*/
SELECT  *
FROM [Sales].[SalesOrderHeader]
WHERE [TotalDue] > 10000;
GO


/*
	check the DMV
*/
SELECT	[size_in_bytes],
		[cacheobjtype],
		[objtype],
		[plan_handle]
FROM [sys].[dm_exec_cached_plans];


/*
	find plan handle based on query text
*/
SELECT	[cp].[size_in_bytes],
		[cp].[cacheobjtype],
		[cp].[objtype],
		[cp].[plan_handle],
		[dest].[text]
FROM [sys].[dm_exec_cached_plans] [cp]
CROSS APPLY [sys].[dm_exec_sql_text]([cp].[plan_handle]) [dest]
WHERE [dest].[text] LIKE '%SalesOrderHeader%';


/*
	get the plan
*/
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
SELECT	[cp].[size_in_bytes],
		[cp].[cacheobjtype],
		[cp].[objtype],
		[cp].[plan_handle],
		[dest].[text], 
		[plan].[query_plan]
FROM [sys].[dm_exec_cached_plans] [cp]
CROSS APPLY [sys].[dm_exec_sql_text]([cp].[plan_handle]) [dest]
CROSS APPLY [sys].[dm_exec_query_plan]([cp].[plan_handle]) [plan]
WHERE [dest].[text] LIKE '%SalesOrderHeader%'
OPTION(MAXDOP 1, RECOMPILE);


/*
	query dm_exec_text_query_plan for large plans
	not clickable link in output
	how to view easily?
*/
SELECT	[dbid],
		[query_plan]
FROM [sys].[dm_exec_text_query_plan](
0x06000600202C8629C0A2D1968B00000001000000000000000000000000000000000000000000000000000000
, 0, -1)




/*
	Look at cardinality estimate issues 
	Run estimated and note estimate
	Run actual and compare...then view in PE
*/
USE [AdventureWorks2014];
GO

DECLARE @Column INT = 2;
DECLARE @Value INT =5;

SELECT [h].[SalesOrderID], [d].[SalesOrderDetailID], [d].[ProductID], [h].[CustomerID], [h].[TerritoryID], [h].[ShipMethodID]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Sales].[SalesOrderHeader] [h] on [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE CHOOSE(@Column, [h].[TerritoryID], [h].[ShipMethodID]) = @Value
GO
