/*============================================================================
  File:     2_CardinalityEstimates.sql

  SQL Server Versions: 2008, 2008R2, 2012, 2014
------------------------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com
  
  (c) 2016, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

/*
	Check for CE issue
*/
USE [AdventureWorks2016E];
GO

SET STATISTICS IO ON;

DECLARE @StartDate SMALLDATETIME, @EndDate SMALLDATETIME

SET @StartDate = '20160801'
SET @EndDate = '20160831'


	DECLARE @ProdHx TABLE (ProductID INT, ProductName VARCHAR(80))

	INSERT INTO @ProdHx (ProductID, ProductName)
	SELECT p.productID, p.name
	FROM Production.TransactionHistoryB [th] 
	JOIN Production.ProductB p on [p].ProductID = [th].ProductID
	WHERE [th].[TransactionDate] between @StartDate and @EndDate
	OPTION (MAXDOP 1)

	SELECT [p].[ProductID],[p].[StandardCost], [p].[ListPrice], [pi].[Quantity], [p].[Name]
	FROM [Production].[ProductB] [p] 
	JOIN [Production].[ProductInventoryB] [pi] ON [p].[ProductID] = [pi].[ProductID] 
	JOIN @ProdHx [ph] ON [p].[ProductID] = [ph].[ProductID]
	WHERE [p].[SellEndDate] IS NOT NULL
	AND [pi].[Quantity]  < 1000
	OPTION (MAXDOP 1)


/*
	run with above, force a recompile
*/
	SELECT [p].[ProductID],[p].[StandardCost], [p].[ListPrice], [pi].[Quantity], [p].[Name]
	FROM [Production].[ProductB] [p] 
	JOIN [Production].[ProductInventoryB] [pi] ON [p].[ProductID] = [pi].[ProductID] 
	JOIN @ProdHx [ph] ON [p].[ProductID] = [ph].[ProductID]
	WHERE [p].[SellEndDate] IS NOT NULL
	AND [pi].[Quantity]  < 1000
	OPTION (RECOMPILE,MAXDOP 1)



/*
	run with above,
	using trace flag 2453
	SQL Server 2012 SP2 
	SQL Server 2014 CU3
	https://support.microsoft.com/en-us/kb/2952444
	http://sqlperformance.com/2014/06/t-sql-queries/table-variable-perf-fix
*/
	DBCC TRACEON (2453)
	
	SELECT [p].[ProductID],[p].[StandardCost], [p].[ListPrice], [pi].[Quantity], [p].[Name]
	FROM [Production].[ProductB] [p] 
	JOIN [Production].[ProductInventoryB] [pi] ON [p].[ProductID] = [pi].[ProductID] 
	JOIN @ProdHx [ph] ON [p].[ProductID] = [ph].[ProductID]
	WHERE [p].[SellEndDate] IS NOT NULL
	AND [pi].[Quantity]  < 1000
	OPTION (MAXDOP 1)
	
	DBCC TRACEOFF (2453)



/*
	can also try a temp table
*/	

	DECLARE @StartDate SMALLDATETIME, @EndDate SMALLDATETIME

	SET @StartDate = '20160801'
	SET @EndDate = '20160831'

	SELECT p.productID, p.name
	INTO #ProdHX
	FROM Production.TransactionHistoryB [th] 
	JOIN Production.ProductB p on [p].ProductID = [th].ProductID
	WHERE [th].[TransactionDate] between @StartDate and @EndDate
	ORDER BY p.ProductID
	OPTION (MAXDOP 1)

	SELECT [p].[ProductID],[p].[StandardCost], [p].[ListPrice], [pi].[Quantity], [p].[Name]
	FROM [Production].[ProductB] [p] 
	JOIN [Production].[ProductInventoryB] [pi] ON [p].[ProductID] = [pi].[ProductID] 
	JOIN #ProdHX [ph] ON [p].[ProductID] = [ph].[ProductID]
	WHERE [p].[SellEndDate] IS NOT NULL
	AND [pi].[Quantity]  < 1000
	OPTION (MAXDOP 1)

	DROP TABLE #ProdHX

/*
	run with above,
	try a temp table and add a NCI
*/	
	SELECT p.productID, p.name
	INTO #ProdHX2
	FROM Production.TransactionHistoryB [th] 
	JOIN Production.ProductB p on [p].ProductID = [th].ProductID
	WHERE [th].[TransactionDate] between @StartDate and @EndDate
	ORDER BY p.ProductID
	OPTION (MAXDOP 1)

	CREATE NONCLUSTERED INDEX NCI1 ON #ProdHX2 (ProductID)

	SELECT [p].[ProductID],[p].[StandardCost], [p].[ListPrice], [pi].[Quantity], [p].[Name]
	FROM [Production].[ProductB] [p] 
	JOIN [Production].[ProductInventoryB] [pi] ON [p].[ProductID] = [pi].[ProductID] 
	JOIN #ProdHX2 [ph] ON [p].[ProductID] = [ph].[ProductID]
	WHERE [p].[SellEndDate] IS NOT NULL
	AND [pi].[Quantity]  < 1000
	OPTION (MAXDOP 1)

	DROP TABLE #ProdHX2

/*
	update the SP...
*/

/*
	check CE in the plan
	use TF 9481 to revert to old CE (if DB in 120 or 130 compat)
	use TF 2312 to get new CE (if DB in <=100 compat)
*/
SELECT [Name], [compatibility_level]
FROM [sys].[databases]
WHERE [Name] = 'AdventureWorks2016E'

DECLARE @StartDate SMALLDATETIME, @EndDate SMALLDATETIME

SET @StartDate = '20160801'
SET @EndDate = '20160831'


	DECLARE @ProdHx TABLE (ProductID INT, ProductName VARCHAR(80))

	INSERT INTO @ProdHx (ProductID, ProductName)
	SELECT p.productID, p.name
	FROM Production.TransactionHistoryB [th] 
	JOIN Production.ProductB p on [p].ProductID = [th].ProductID
	WHERE [th].[TransactionDate] between @StartDate and @EndDate
	OPTION (MAXDOP 1)

	DECLARE @ProdHx2 TABLE (ProductID INT, ProductName VARCHAR(80))

	INSERT INTO @ProdHx2 (ProductID, ProductName)
	SELECT p.productID, p.name
	FROM Production.TransactionHistoryB [th] 
	JOIN Production.ProductB p on [p].ProductID = [th].ProductID
	WHERE [th].[TransactionDate] between @StartDate and @EndDate
	OPTION (QUERYTRACEON 9481, MAXDOP 1)

