/*================================================================
  File:     3_NestedLoops.sql

  Summary:  Nested loop example with a large number of 
			random IOs on the inner loop

  SQL Server Versions: SQL Server 2012, 2014, 2016
------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com

  (c) 2016, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
=================================================================*/
USE [AdventureWorks2016E];
GO

/*
	General query that uses a nested loop
*/
SELECT [d].[ProductID], SUM([d].[OrderQty])
FROM [Sales].[SalesOrderDetailB] [d]
JOIN [Sales].[SalesOrderHeaderB] [h] ON [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [d].[ProductID] = 898
GROUP BY [d].[ProductID];
GO

DBCC SHOW_STATISTICS ('Sales.SalesOrderDetailB', IX_SalesOrderDetailB_ProductID);
GO

select top 100 *
FROm
[Sales].[SalesOrderHeaderB]

select top 100 *
from
[Sales].[SalesOrderDetailB]

/*
	Create SP
*/
DROP PROCEDURE IF EXISTS [Sales].[usp_GetProductCount];
GO

CREATE PROCEDURE [Sales].[usp_GetProductCount]
	@ProductID INT
AS	

	SELECT [d].[ProductID], SUM([d].[OrderQty])
	FROM [Sales].[SalesOrderDetailB] [d]
	JOIN [Sales].[SalesOrderHeaderB] [h] ON [d].[SalesOrderID] = [h].[SalesOrderID]
	WHERE [d].[ProductID] = @ProductID
	GROUP BY [d].[ProductID];

GO


/*
	Run SP a few times
*/
DBCC FREEPROCCACHE;
GO

EXEC [Sales].[usp_GetProductCount] 898;
GO

EXEC [Sales].[usp_GetProductCount] 1771;
GO

EXEC [Sales].[usp_GetProductCount] 1934;
GO

/*
	Need to look at more than just the plan...
*/
SET STATISTICS IO ON;
SET STATISTICS TIME ON;
EXEC [Sales].[usp_GetProductCount] 898;
GO

EXEC [Sales].[usp_GetProductCount] 1771;
GO

EXEC [Sales].[usp_GetProductCount] 1934;
GO


/*
	What if I run the plans separately?
*/
EXEC [Sales].[usp_GetProductCount] 898 WITH RECOMPILE;
GO
EXEC [Sales].[usp_GetProductCount] 1934 WITH RECOMPILE;
GO
/*
	note the IO here:

*/

EXEC sp_recompile '[Sales].[usp_GetProductCount]';
GO

/*
	can we make this better?
*/
EXEC sp_SQLskills_helpindex 'Sales.SalesOrderDetailB';
GO


CREATE NONCLUSTERED INDEX [IX_SalesOrderDetailB_ProductID] 
	ON [Sales].[SalesOrderDetailB](
		[ProductID] ASC
	)
	INCLUDE (
		[OrderQty]
	)
WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = ON, IGNORE_DUP_KEY = OFF, 
DROP_EXISTING = ON, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) 
ON [PRIMARY];
GO

/*
	changing THIS index may not change what's in the cache
	(will only invalidate a plan if it's used in the plan)
*/
EXEC [Sales].[usp_GetProductCount] 1934;
GO

/*
	force the recompile
*/
EXEC [Sales].[usp_GetProductCount] 898 WITH RECOMPILE;
GO
EXEC [Sales].[usp_GetProductCount] 1934 WITH RECOMPILE;
GO


/*
	clean up
*/
DROP PROCEDURE [Sales].[usp_GetProductCount];
GO


/*
	 Why is estimated rows <> actual rows for the key lookup?
*/
SELECT [d].[ProductID], SUM([d].[OrderQty])
FROM [Sales].[SalesOrderDetailB] [d]
JOIN [Sales].[SalesOrderHeaderB] [h] ON [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [ProductID] = 727
GROUP BY [d].[ProductID]


